/* File: vararg.c
   Calculate areas of different shapes using a function 
   with a variable number of arguments. */
#include <stdio.h>
#include <stdarg.h>   /* for va_list, va_start(), va_arg(), va_end() */
#include <math.h>     /* for M_PI */

/* define macros for circle, square, and rectangle */
#define CIRCLE        1
#define SQUARE        2
#define RECTANGLE     3

/* function area() with a variable number of arguments */
double area (int type, ...) {
    va_list ap;           /* variable used for handling ... */
    double radius;        /* radius of a circle */
    double side;          /* side of a square */
    double length, width; /* length and width of a rectangle */
    double a = 0.0;       /* area of a shape, default is 0 */

    va_start(ap, type); /* use the argument type to start ap */
    /* depending on the type of shape,
       obtain the passed argument(s) and calculate the area */
    switch(type) {
        case CIRCLE:
          /* for a circle, obtain the passed radius  */
          radius = va_arg(ap, double); 
          a = M_PI*radius*radius;
          break;
        case SQUARE:
          /* for a square, obtain the passed side */
          side = va_arg(ap, double);
          a = side*side;
          break;
        case RECTANGLE:
          /* for a rectanle, obtain the passed length and width */
          length = va_arg(ap, double);
          width  = va_arg(ap, double);
          a = length*width;
          break;
        default:
          /* in case a mistake is made by passing an unknown shape */
          printf("Error: unknown shape\n");
          break;
    }
    va_end(ap); /* end the processing of the variable number of arguments*/

    return a;
}

int main() {
    /* The unit for lengths is meter */
    double radius = 2.0;              /* radius of a circle */
    double side = 2.0;                /* side of a square */
    double length = 3.0, width = 2.0; /* length and width of a rectangle */
    double a;                         /* area of a shape */

    /* calculate and display areas for shapes circle, square, and rectangle */
    a = area(CIRCLE, radius);
    printf("The area of circle with radius 2 m is %.2f (m^2)\n", a);
    a = area(SQUARE, side);
    printf("The area of square with side 2 m is %.2f (m^2)\n", a);
    a = area(RECTANGLE, length, width);
    /* A single long line of output is broken into two printf() statements */
    printf("The area of rectangle with length 3 m and width 2 m");
    printf(" is %.2f (m^2)\n", a);
    return 0;
}
