/* File: bisect2.c 
   Find roots for the projectile function using the bisection method 
   in the range [1, 200] with the initial projection speed of 35m/s and
   projection angles 15 degrees and 45 degrees. */
#include <stdio.h>
#include <math.h>        /* for fabs() */
#include <float.h>       /* for FLT_EPSILON */

#define M_G   9.81       /* gravitational acceleration constant g */

/* function prototype for bisect2() */ 
int bisect2(double *x, double x0, double xf, double tolerance,
            double (*func)(double, void*), void *param);

/* function for the y-coordinate of the projectifle with the projection angle
   as a parameter */
double traj(double x, void *param) {
   double theta0,        /* projection angle */
          v0,            /* initial velocity */
          y;             /* y coordinate */

   theta0 = *(double*)param; /* calculate the projection angle in radians */
   v0 = 35.0;            /* set the initial velocity */
   /* calculate the y coordinate */
   y = tan(theta0)*x - M_G*x*x/(2*v0*v0*cos(theta0)*cos(theta0));
   return y;
}

int main() {
    /* find the root of projectile function y(x) specified in the range 
    [60.0, 70.0]. use machine epsilon for float type as convergence tolerance */
    double x, x0=1.0, xf=200.0, tolerance=FLT_EPSILON,
           theta0 = 15*M_PI/180; /*  the projection angle in radians */
    int status;

    theta0 = 15*M_PI/180; /*  the projection angle in radians */
    status = bisect2(&x, x0, xf, tolerance, traj, &theta0);
    if(status < 0)  /* calling bisect2() failed */
        printf("Error: bisect2() failed, status = %d\n", status);
    else            /* calling bisect2() successfully */
        printf("x = %f m, traj(x) = %g\n", x, traj(x, &theta0));

    theta0 = 45*M_PI/180; /* the projection angle in radians */
    status = bisect2(&x, x0, xf, tolerance, traj, &theta0);
    if(status < 0)  /* calling bisect2() failed */
        printf("Error: bisect2() failed, status = %d\n", status);
    else            /* calling bisect2() successfully */
        printf("x = %f m, traj(x) = %g\n", x, traj(x, &theta0));
    return 0;
}
