/* File: acceloption.c
   Calculate accelerations with time from the command line. 
   When the command line has an option -p, print force. */
#include <stdio.h>
#include <stdlib.h>     /* for exit() */
#define TRUE  1         /* define boolean value true */
#define FALSE 0         /* define boolean value false */
#define M_G   9.81      /* gravitational acceleration constant g */

double force(double t) { /* function for force */
    double p;
    p = 4*(t-3)+20;
    return p;
}
int main(int argc, char **argv) {
    char *s, *cmd;        /* declare variables s and cmd */
    int p_option = FALSE; /* default, no -p option */
    double a, mu, m, p, t; /* declare a, mu, m, p, t for calculating accel */

    cmd = argv[0];        /* save cmd in argv[0] */
    if(argc < 2 || argc > 3) {  /* not enough or extra argument */
       printf("Usage: %s [-p] time \n", cmd);
       printf("       -p will print out the force.\n");
       exit(EXIT_FAILURE);
    }

    argc--; argv++;  /* skip the command, the 1st one in the argument list */ 
    /* process each argument beginning with - */
    while(argc > 0 && **argv == '-') {
       /* empty space is not valid option */
       /* use a for-loop to handle multiple options followed
          with '-' such as as '-p', "s = argv[0]+1" skips '-'  */
       for(s = argv[0]+1; *s != '\0'; s++) { 
         switch(*s) {
           case 'p':               /* if *s is 'p' in '-p' */
             p_option = TRUE;      /* print out force  */
             break;
           default:
             printf("Warning: invalid option %c\n", *s);
             printf("Usage: %s [-p] time \n", cmd);
             printf("       -p will print out the force.\n");
             exit(EXIT_FAILURE);
          }
       }
       argc--; argv++;          /* move to the next argument */
    }
    /* convert string to double for time entered in command line,
       then calculate and display the acceleration */
    t = atof(argv[0]);          /* convert a string to double */
    mu = 0.2; m = 5.0; p = force(t);
    a = (p-mu*m*M_G)/m;
    if(p_option)                /* if has -p option , print force */
      printf("Force p = %f\n", p);
    printf("Acceleration a = %f\n", a);
    return 0;
}
