/* File: accelfile.c
   Write accelerations into a file  with the user specified file name */
#include <stdio.h>  /* for fopen(), fclose(), fprintf() */
#include <stdlib.h> /* for exit() */
#define M_G   9.81  /* gravitational acceleration constant g */

char dfilename[] = "accel.txt"; /* default file name */

int main(int argc, char **argv) {
    /* declaration and initialization */
    double a, mu=0.2, m=5.0, t0=0.0, tf=10.0, step=1.0, p, t;
    int i, n;      
    FILE *stream;          /* a file stream */
    char *cmd, *filename;  /* cmd and file name */ 

    cmd = argv[0];         /* save the command */
    if(argc >= 3){         /* extra argument */
        fprintf(stderr, "Usage: %s [filename]\n", cmd);
        exit(EXIT_FAILURE);
    }
    /* get the user specified or default file name */
    filename = (argc == 2 ? argv[1]:dfilename); 
    /* open the file for writing */
    stream = fopen(filename, "w"); 
    if(stream == NULL) {
        printf("Error: cannot open '%s'\n", filename);
        exit(EXIT_FAILURE);
    }
    n = (tf - t0)/step + 1;       /* number of points, 11 */
    for(i=0; i<n; i++) {
       t = t0 + i*step;           /* calculate value t */
       p = 4*sin(t-3)+20;         /* use t to calculate p */
       a = (p-mu*m*M_G)/m;        /* calculate a */
       fprintf(stream, "%f\n", a);/* write acceleration into the file */ 
    }
    fclose(stream);               /* close the file stream */
    printf("Done! Finished writing data to file %s\n", filename);
    return 0;
}
