/* File: rwstudent.c
   Read student information from file "rwstudent.data" and
   write it into another file "rwstudentNew.data" */
#include <stdio.h>  /* for fopen(), fclose(), fprintf(), fscanf(), printf() */

struct Birthday {            /* define structure for birthday */
    short day, month, year;
};

typedef struct Student {     /* define structure for student */
    int id;
    char name[32];  
    struct Birthday birthday;
    double gpa;
} student_t;

/* read student information in filename into the buffer pointed by sp */
int readData(student_t *sp, const char *filename) {
    FILE *stream;

    stream = fopen(filename, "r");
    if(stream == NULL) {
        fprintf(stderr, "Error: cannot open file '%s' for reading\n", filename);
        return -1;
    }
    fscanf(stream, "%d%s%hd%hd%hd%lf", &sp->id, sp->name,
               &sp->birthday.day, &sp->birthday.month, &sp->birthday.year,
               &sp->gpa);
    fclose(stream);
    return 0;
}

/* write student information in the buffer pointed by sp into filename */
int writeData(const student_t *sp, const char *filename) {
    FILE *stream;

    stream = fopen(filename, "w"); 
    if(stream == NULL) {
        fprintf(stderr, "Error: cannot open file '%s' for writing\n", filename);
        return -1;
    }
    fprintf(stream, "%d %s %hd %hd %hd %g\n", sp->id, sp->name,
        sp->birthday.day, sp->birthday.month, sp->birthday.year, sp->gpa);
    fclose(stream);
    return 0;
}

int main() {
    const char *rfilename = "rwstudent.data";    /* read this data file */
    const char *wfilename = "rwstudentNew.data"; /* write into ths file */
    student_t s;    /* declare a student_t to hold the information */
    int status;     /* status for writing */

    readData(&s, rfilename);           /* read data in rfilename into s */
    status = writeData(&s, wfilename); /* write data from s into wfilename */ 
    /* demonstrate how a return value should be used, 
       give a message in stdout  */ 
    if(status)
       printf("writeData() failed\n");
    else
       printf("writeData() is successful\n");
    return 0;
}      
