/************************************************************
* File: studentfile.c
* This program tests the GPA library with data in a file
************************************************************/
#include <stdio.h>
#include <stdlib.h>       /* for exit(EXIT_FAILURE) */
#include "studentfile.h"  /* header file for GPA library */

int main() {
    student_t *head, *sp; /* head of the list and a single student */
    double gpa;           /* GPA value */
    struct Birthday b;    /* b for birthday */
    int num; /* number of students with GPA above a specified value */
    const char filename[] = "student.data"; /* student data file */

  
    /* read student data file */
    head = gpa_readData(filename);
    if(head == NULL) {
        fprintf(stderr, "Error: failed to read file '%s'\n", filename);
        exit(EXIT_FAILURE);
    }
    printf("The total number of students is %d\n", gpa_totnum(head));
    gpa = gpa_averageGPA(head); /* GPA for all students */
    printf("The GPA for all students is %.2f\n", gpa);
    sp = gpa_highestGPA(head); /* find the student with the highest GPA */
    b = gpa_getBirthday(sp);   /* get the birthday of this student */
    printf("%s who was born on %s %d, %d has the highest GPA %.2f\n",
           gpa_getName(sp), gpa_monthString(b.month), b.day, b.year, 
           gpa_getGPA(sp));
    printf("The student with the lowest GPA:\n");
    sp = gpa_lowestGPA(head);  /* find the student with the lowest GPA */
    gpa_printStudent(sp);      /* print infor for this student */ 
    /* find the student with ID 103 and named 'Peter' */
    sp = gpa_find(head, 103, "Peter");
    if(sp) {  /* if Peter is in the list, get and print his GPA */
      printf("Peter's GPA is %.2f\n", gpa_getGPA(sp));
    }
    printf("Students with GPA 3.0 or above:\n");
    /* get the number of students with GPA above 3.0 and print them out */
    num = gpa_aboveGPA(head, 3.0); 
    printf("%d students with GPA 3.0 or above.\n", num);
    gpa_sortGPA(head);   /* sort students in the list based on GPA */
    printf("The student record sorted in descending GPA:\n");
    gpa_printList(head); /* print info for students in the sorted list */
    /* write the student data file with sorted GPA */
    gpa_writeData(head, "studentNew.data");
    gpa_clearList(head); /* clear the memory allocated for the data */
    return 0;
}
