/************************************************************
* File: studentfile.h
* This header file contains macros, data structures and
* function prototypes for the GPA library using a linked list. 
* Macro names with prefix GPA_,  variable names with prefix gpa_, 
* symbols Birthday, ListStudent, student_t, StudentNode are reserved 
* for this lib.
************************************************************/
#ifndef STUDENTFILE_H
#define STUDENTFILE_H

/* define macros for building a binary GPA library */
#if defined(_WIN32)  /* for compilation using VC++ in Windows */
#define EXTERN extern __declspec(dllexport)
#else                /* for compilation in Unix or run in Ch */
#define EXTERN extern 
#endif

/* define boolean numbers */
#define TRUE 1
#define FALSE 0

struct Birthday {
    short day, month, year;
};

struct ListStudent {
    int id;
    char *name;  
    struct Birthday birthday;
    double gpa;
    struct ListStudent *next;
};

/* define new data types student_t for compatibility of the 
   previous code studentfile.c */
typedef struct ListStudent student_t; 
typedef struct ListStudent *StudentNode; /* for linked list */

/* functions for a single student */
EXTERN void gpa_printStudent(StudentNode sn);
EXTERN int gpa_getId(StudentNode sn);
EXTERN char *gpa_getName(StudentNode sn);
EXTERN struct Birthday gpa_getBirthday(StudentNode sn);
EXTERN double gpa_getGPA(StudentNode sn);
EXTERN void gpa_setGPA(StudentNode sn, double gpa);
/* functions for a linked list */
EXTERN StudentNode gpa_createStudent(int id, const char *name, 
                     struct Birthday b, double gpa);
EXTERN void gpa_freeStudent(StudentNode sn);

/* functions for a list of students */
EXTERN int gpa_totnum(StudentNode head);
EXTERN void gpa_printList(StudentNode head);
EXTERN double gpa_averageGPA(StudentNode head);
EXTERN StudentNode gpa_highestGPA(StudentNode head);
EXTERN StudentNode gpa_lowestGPA(StudentNode head);
EXTERN StudentNode gpa_find(StudentNode head, int id, const char *name);
EXTERN int gpa_aboveGPA(StudentNode head, double gpa);
EXTERN void gpa_sortGPA(StudentNode head);
EXTERN StudentNode gpa_readData(const char *filename);
EXTERN int gpa_writeData(StudentNode head, const char *filename);
EXTERN void gpa_clearList(StudentNode head);
/* functions for a linked list */
EXTERN void gpa_prepend(StudentNode *head, StudentNode sn);   
EXTERN void gpa_append(StudentNode *head, StudentNode sn);   
EXTERN int gpa_insert(StudentNode *head, StudentNode sn, int pos);   
EXTERN void gpa_remove(StudentNode *head, StudentNode sn);   

/* utility functions */
EXTERN const char *gpa_monthString(int month);

/* load code from file studentlistfunc.c for Ch */
#ifdef _CH_
#pragma importf "studentlistfunc.c"
#endif

#endif /* STUDENTFILE_H */
