/* File: sqrtr.c 
   Function sqrtx() based on Newton's method for calculating a square root.
   It can handle -x, NaN, -Inf, Inf 
   Use isnan(), NAN, INFINITY in C99 to implement sqrtx().
   Compile in gcc, use command 'gcc -std=c99 sqrtr.c'  */
#include <stdio.h>
#include <math.h>  /* for fabs() */
#include <float.h> /* for FLT_EPSILON */

#define N 100      /* the maximum number of iteration */

double sqrtx(double x) {
   int i;
   double x0, x1, x2;

   /* handle negative and metanumbers */
   if(isnan(x) || x<0.0)
     return NAN;
   else if(x == INFINITY)
     return INFINITY;

   x0 = x;         /* an initial guess for x0 */
   x1 = x0;                   /* set x1 to x0 */
   x2 = (x1+x/x1)/2.0;        /* recursive formula */
   for(i = 1; i <= N && fabs(x2-x0) >= FLT_EPSILON; i++) {
      x0 = x1;                /* update x0  for fabs(x2-x0) */
      x2 = (x1+x/x1)/2.0;     /* Newton's recursive formula */
      x1 = x2;     /* update value x1 for n
      x1 = x2;     /* update value x1 for next iteration */
   }

   if(i < N) {     /* number of iteration is less than N */
     return x2;    /* return the square root */
   }
   else {          /* number of iteration equals N */
     /* printf("sqrtx() failed to converge\n"); */
     return NAN;   /* return NaN if failed to converge */
   }
}

int main() {
   double a;

   a = 3.0;       /* for sqrtx(3.0) */
   printf("sqrtx(%.2f) = %f\n", a, sqrtx(a));
   a = -3.0;      /* for sqrtx(-3.0) */
   printf("sqrtx(%.2f) = %f\n", a, sqrtx(a));
   a = NAN;       /* for sqrtx(NaN) */
   printf("sqrtx(%.2f) = %f\n", a, sqrtx(a));
   a = -INFINITY; /* for sqrtx(-Inf) */
   printf("sqrtx(%.2f) = %f\n", a, sqrtx(a));
   a = INFINITY;  /* for sqrtx(Inf) */
   printf("sqrtx(%.2f) = %f\n", a, sqrtx(a));
   return 0;
}
