/* File: ccomplex.h 
   A portable header file for using complex in C99 and C++.
   (1) The following features are added in C++ 
       (a) complex types double_complex and float_complex 
       (b) imaginary number I
       (c) functions creal(), cimag(), cabs(), and carg()
   (2) The following features are added in C99 and Ch
       (a) types double_complex and float_complex 
   (3) The following feature is added in C99 
       (a) function polar()
*/
#ifndef CCOMPLEX_H 
#define CCOMPLEX_H

#ifdef __cplusplus  /* for C++ */
  #include <math.h>                       /* for math functions */
  #include <iostream>                     /* for using namespace std */
  #include <complex>                      /* for complex class */
  using namespace std;                    /* for complex I/O */
  typedef complex<double> double_complex; /* double complext type */
  typedef complex<float>  float_complex;  /* float complex or complex type */
  #define I double_complex(0.0, 1.0)      /* define pure imaginary number */
  #define creal real                      /* define creal() */
  #define cimag imag                      /* define cimag() */
  #define cabs  abs                       /* define cabs() */
  #define carg  arg                       /* define carg() */
  #define casin(z)  (-I*log(I*(z)+sqrt(1.0-(z)*(z)))) /* define cacos() */
  #define cacos(z)  (-I*log((z)+I*sqrt(1.0-(z)*(z)))) /* define cacos() */

#else              /* for C99 and Ch */
  #include <tgmath.h>                     /* type generic functions */
  #include <complex.h>                    /* complex numbers */
  typedef float complex float_complex;    /* define float complex type */
  typedef double complex double_complex;  /* define double complex type */
  /* function polar() is available in C++ and Ch. Define it for C99 */
  #ifndef _CH_    
  #define polar(r, theta)  ((r)*cos(theta)+I*(r)*sin(theta)) 
  #endif 
#endif

#endif /* end CCOMPLEX_H */
