/* File: cexample.c (run in C99 and Ch only)
   Compile in gcc, use command 'gcc -std=c99 cexample.c' 
   Handle complex numbers: declaration, complex constructor, 
   real and imaginary parts, modulus, phase angle, input and output */
#include <stdio.h>
#include <complex.h> /* for complex, I, creal(), cimag(), cabs(), carg() */

int main() {
    double complex z;       /* declare z as double complex */
    double *preal, *pimag;  /* pointer to real and imaginary parts */

    /* use creal() and cimag() to get real and imag parts of z */
    z = 3.0 + I*4.0;        /* construct complex number 3+i4 */
    printf("z = complex(%f, %f)\n", creal(z), cimag(z));
    printf("modulus(z) = %f, phase angle (z) = %f\n", cabs(z), carg(z));
  
    /* use pointers to double to change real and imag parts of z */
    preal = (double *)&z;   /* preal points to real part of z */
    pimag = preal+1;        /* pimag points to imag part of z */
    *preal = 10.0;          /* real(z) = 10.0 */
    *pimag = 20.0;          /* imag(z) = 20.0 */
    printf("z = complex(%f, %f)\n", *preal, *pimag);
    printf("z = complex(%f, %f)\n", creal(z), cimag(z));

    /* use preal and pimag to access z for scanf() */
    printf("Please type in real and imaginary parts of complex number:\n");
    scanf("%lf%lf", preal, pimag); 
    printf("z = complex(%f, %f)\n", creal(z), cimag(z));
    return 0;
}
