/* File: projectilestruct.cpp 
   Plot a projectile without using global variable */
#include <math.h>       // for sin(), cos(), tan()
#include <chplot.h>     // for CPlot

#define M_G 9.81        // gravitational constant g
#define N 100           // number of points for plotting

struct InitCond {       // define struct InitCond
   double theta0;       // member for projection angle 
   double v0;           // member for initial velocity
};

double traj(double x, void *param) {
   double theta0,       // variable for projection angle 
          v0,           // variable for initial velocity
          y;            // y coordinate 
   struct InitCond *initcptr;

   initcptr = (struct InitCond *)param;  // cast to proper pointer type
   theta0 = initcptr->theta0;            // access value from the pointer 
   v0 = initcptr->v0;
   y = tan(theta0)*x - M_G*x*x/(2*v0*v0*cos(theta0)*cos(theta0));
   return y;
}

int main() {
   struct InitCond initc; // variables for projection angle and initial velocity
   double x0=0, xf;
   CPlot plot;


   initc.theta0 = 15*M_PI/180;    // projection angle in rad
   initc.v0 = 35.0;               // initial velocity
   xf = initc.v0*initc.v0*sin(2*initc.theta0)/M_G;    // the distance
   plot.title("A projectile with the initial projection angle 15 degrees");
   plot.label(PLOT_AXIS_X, "x (m)");
   plot.label(PLOT_AXIS_Y, "y (m)");
   plot.funcp2D(x0, xf, N, traj, &initc); // address of initc is passed
   plot.plotting();
}
