/* File: accelcharrayvla.ch 
   Calculate and plot accels using variable length computational arrays */
#include <numeric.h> // for lindata(), sin(), and 'array' type qualifier
#include <chplot.h>  // for plotxy()

#define M_G 9.81     // gravitational acceleration constant g
#define N1 11        // number of points for plotting 
#define N2 100       // number of points for plotting 

/* function passes variable length arrays */
void accelvla(int n, array double a[n]) {
    array double t[n], p[n]; // declare deferred-shape arrays t and p
    double mu, m, t0, tf;    // declare variables

    mu = 0.2;                // mu 
    m = 5.0;                 // mass 
    t0 = 0.0;                // initial time 0 second 
    tf = 10.0;               // final time 10 seconds 
    lindata(t0, tf, t);      // fill array t from t0 to tf linearly 
    p = 4*(sin(t-3))+20;     // p = 4sin(t-3)+20
    a = (p-mu*m*M_G)/m;      // a = (p-mu*m*M_G)/m 
    plotxy(t, a, n, "Acceleration Plot", "time (s)", "accel (m/s^2)");
}

int main() {
    array double a1[N1], a2[N2];  // declare arrays with different size

    accelvla(N1, a1);             // pass array a1 with N1 elements
    accelvla(N2, a2);             // pass array a2 with N2 elements
    return 0;
}
