/* File: oderk.ch
   Solve an ordinary different equation, the van der Pol equation,
   using function oderk() in Runge Kutta method */
#include <chplot.h>    // plotxy()
#include <numeric.h>   // header file numerical functions, for oderk()

#define NVAR 2         // number of dependent variables y1 and y2,
                       // or order of the equation
#define POINTS 256     // maximum number of points for ODE

/* function for differential equation */
void func(double t, double y[], double dydt[], void *param) { 
    double mu;            // mu in the van der Pol equation 

    /* obtain mu passed from call oderk(func, t0, tf, y0, &mu, t, y); */
    mu = *(double*)param; 
    dydt[0] = y[1];                       // 1st equation in the ODE  
    dydt[1]=mu*(1-y[0]*y[0])*y[1] - y[0]; // 2nd equation in the ODE
}

int main() {
    double t0=1, tf=30, y0[NVAR] = {1, 0}; // initial condition
    /* arrays to keep the results from the ODE solver oderk() */
    double t[POINTS], y[NVAR][POINTS];
    double mu = 2;                         //  declare and initialize mu
    
    /* call oderk() to solve the van der Pol equation with t and y */
    oderk(func, t0, tf, y0, &mu, t, y); 
    /* plot y1 and y2 versus t, 2D array y contains y[0] and y[1] */
    plotxy(t, y, "The solution for the van der Pol equation", "t (seconds)",
           "y1 and y2");
    return 0;
}
