/* File: accelcharrayreturn.ch 
   Calculate and plot accelerations using a function returning arrays */
#include <numeric.h> // for lindata(), sin(), and 'array' type qualifier
#include <chplot.h>  // for plotxy()

#define M_G 9.81     // gravitational acceleration constant g
#define N1 11        // number of points for plotting
#define N2 100       // number of points for plotting

/* function passes variable length arrays using assumed-shape array
   and returns array of variable length */
array double accelreturnarray(array double t[:])[:] {
    int n = (int)shape(t);   // obtain the array size
    array double a[n], p[n]; // declare deferred-shape arrays a and p
    double mu, m;            // declare mu and mass m

    mu = 0.2;                // mu
    m = 5.0;                 // mass
    p = 4*(sin(t-3))+20;     // p = 4sin(t-3)+20
    a = (p-mu*m*M_G)/m;      // a = (p-mu*m*M_G)/m
    return a;                // return array a
}

int main() {
    /* declare arrays of different sizes */
    array double t1[N1], t2[N2]; 
    array double a1[N1], a2[N2];
    double t0, tf;

    t0 = 0.0;                // initial time 0 second
    tf = 10.0;               // final time 10 seconds
    lindata(t0, tf, t1);     // fill array t1 from t0 to tf linearly
    /* pass array t1 to function which returns acclerations in an array */
    a1 = accelreturnarray(t1); 
    plotxy(t1, a1, "Acceleration Plot", "time (s)", "accel (m/s^2)");

    lindata(t0, tf, t2);     // fill array t2 from t0 to tf linearly
    /* pass array t2 to function which returns acclerations in an array */
    a2 = accelreturnarray(t2);
    plotxy(t2, a2, "Acceleration Plot", "time (s)", "accel (m/s^2)");
    return 0;
}
